
#  ------------------------------------------------------------------------

# Analza hlavnch komponent ----------------------------------------------

#  ------------------------------------------------------------------------

install.packages(c('vegan', 'gclus', 'cluster', 'vegan', 'ade4'))
install.packages(c('cluster', 'permute', 'lattice'))

library(cluster)
library(gclus)
library(vegan)
library(ade4)


# nastaveni cesty
setwd("F:/FSTA_2015/07_cviceni")
getwd()
dir()

lokality<-read.table('lokality.csv', sep=";", header=TRUE, dec=".")

summary (lokality) # popisna statistika



# PCA na celem souboru ----------------------------------------------------

# Priklad 1: --------------------------------------------------------------

# Na dovem souboru lokality provedte analyzu hlavnich komponent
# 1) Jakou matici pouzijete na vstupu?
# 2) Kolik hlavnich komponent budete interpretovat?
# 3) Vykreslete biplot

?rda
# -> 1) pouzijeme matici korelaci 

lokality.pca<-rda (lokality, scale=TRUE) # korelacni matice -> scale=TRUE ; udela standardizaci promennych

lokality.pca

summary(lokality.pca) # Nastaveno skalovani typu 2 - defaultne
                      # Korelacni biplot - kazdy vlastni vector je skalovany na ctverec jeho vlastni hodnoty
                      # Vzdalenost objektu v ordinacnim prostoru neni  aproximaci Euklidovskych vzdalenosti 
                      # ve vicerozmernem prostoru
                      # Uhly mezi promennymi v grafu reflektuji jejich vzajemnou korelaci

# Species scores -> koordinaty "sipek" promennych 
# Site scores -> koordinaty bodu v ordinacnim diagramu

summary(lokality.pca, scaling=1)  # Skalovani typu 1: Vlatsni vektory jsou jsou skalovany na jednotku delky 
                                  # Vzdalenosti mezi objekty jsou primou aproximaci Euklidovskych vzdalenosti 
                                  # ve vicerozmernem prostoru  
                                  # Uhly mezi promennymi v grafu nemaji zadny vyznam

# -> 2) Pocet os, ktere budeme interpreovat zjistime dle podle: 
#       a) Kaiserova-Guttmanova kriteria

?cca.objects # jakou maji strukturu vystupy a jak je lze extrhovat 

eig<-lokality.pca$CA$eig # ziskani vlastnich hodnot

eig[eig > mean(eig)]  # Aplikace Kaiserova-Guttmanova kriteria - inerpretujeme tolik vlastnich hodnot, 
                      # ktere maji vyssi hodnotu nez je prumerna hodnota vlastnich hodnot v souboru

#       a) Broken stick model ->  nahodne deli cast jednotky delky do stejneho poctu casti jako mame pocet 
                                # hlavnich komponent, hodnoty jsou serazeny sestupne a pak jsou srovnavany s 
                                # procenty vycerpane vaiability procenta, ketra jsou vetsi indikuji spravny pocet komponent 

n<-length (eig)
bsm<-data.frame(j=seq(1:n), p=0)
bsm$p<-1/n

for (i in 2:n) {
  bsm$p[i] = bsm$p[i-1]+ (1/(n+1-i))
}

bsm$p<-100*bsm$p/n
bsm


# Graf valstnich hodnot a % variability pro kazdou hlavni komponentu

par(mfrow=c(2,1))
barplot(eig, main="Vlastni hodnoty", col="bisque", las=2)
abline(h=mean(eig), col="red")
legend ("topright", "Prumerna vlastni hodnota", lwd=1, col=2, bty = "n")

barplot (t(cbind(100*eig/sum(eig),bsm$p[n:1])), beside=TRUE, mail="% variability", col=(c("bisque",2)), las=2)
legend ("topright", c("% variability","Broken stick model"), pch=15, col=(c("bisque",2)), bty = "n")

# Kazda metoda ukazuje na jiny pocet hlavnich komponent.

#################################
# jednodusi -> primo vzstup, funkce kreslici predchozi grafy 

source("evplot.R")
evplot(eig)

# -> 3) Vykresleni biplotu
#       Existuji dva typy podle pouziteho skalovani

# Pouziti funkce biplot
?biplot

par(mfrow=c(1,2))
biplot(lokality.pca, scaling = 1, main= "PCA - skalovani 1")
biplot(lokality.pca, main= "PCA - skalovani 2")

# pomoci cleanplot.pca

source("cleanplot.pca")

# S body pro lokality a "sipky"
cleanplot.pca (lokality.pca, point=TRUE)

# pouze s popisem lokalit
cleanplot.pca (lokality.pca)
cleanplot.pca (lokality.pca, ahead=0) # ...bez sipek



#  ------------------------------------------------------------------------

#  Kombinace shlukove analyzy a PCA ---------------------------------------

#  ------------------------------------------------------------------------

# Priklad 2 ---------------------------------------------------------------

# Srovnjete vystup z PCA s vystupe ze shlukove analyzy a zjiste zda je v datech nejaka struktura.
# Objekty shlukujte na zaklade Euklidovske vzdalenosti pomoci Wardovy metody (algoritmu)

lokality.w<-hclust(dist(scale(lokality)), "ward.D") # vypocet shlukove analyzy
                                                    # PZOR -> mozna mate jinou verzi R 
                                                    # -> misto "ward.D" nzpiste jen "ward"
par(mfrow=c(1,1))

plot(lokality.w)            # data rozdelime do 4 shluku
gr<-cutree(lokality.w, k=4) # rozdeleni souboru
grl<-levels(factor(gr))     # vytvoreni popisek 

sit.sc<-scores(lokality.pca, display="sites", scaling=1) # extrahujeme si score pro lokality

# Vykreslime graf z PCA s obarvenymi lokalitami dle prislusnoti do shluku

p<-plot(lokality.pca,display="sites", scaling=1, type="n", main="PCA korelace + shluky" )
abline(v=0, lty="dotted")
abline(h=0, lty="dotted")
for (i in 1:length(grl)) {
  points (sit.sc[gr==i,], pch=(14+i), cex=2, col=i+1)
}
text (sit.sc, row.names(lokality), cex=0.7, pos=3)

# Pridame dendrogram do grafu
ordicluster (p, lokality.w, col="dark grey")
legend(locator(1), paste ("Group", c(1:length(grl))), pch=14+c(1:length(grl)), col=1+c(1:length(grl)), pt.cex=2)


#  ------------------------------------------------------------------------

# Ukol 1 ------------------------------------------------------------------

# Provedte PCA na datovem souboru car_sales.csv

cars<-read.table('car_sales.csv', sep=";", header=TRUE, dec=".")

# data je potreba trochu upravit
names(cars)

cars<-cars[!is.na(cars$X4.year.resale.value),] # X4.year.resale.value obsahuje missingy(chybejici hodnoty) - > musi se umazat jinak to nebude fungovat

# dale si vlozime nazev auta z promenne Model jak nazev radku -> abychom v biplotu meli nazvy aut
rownames(cars)<-cars$Model

rownumber(cars[cars$Model=="Neon",]) # mame tam duplicitu, dva auta maji stejny nazev "Neon", je potreba jej prejmenovat

# prejmenovani se musi udelat tak ze nejdrive je potreba pridat novy nazev do mezi faktory promenne (to jsou nazvy tech aut)
levels(cars$Model) <- c(levels(cars$Model),"Neon_2")
cars["78",2]<-"Neon_2" # pak uz jen jednoduse nazev prejmenujeme

rownames(cars)<-cars$Model # vlozime nazvy aut z promenne Model jako nazvy radku

# Vypocet PCA
dim(cars)

cars.pca<-rda (cars[,3:14], scale=TRUE) # PCA na korelacni matici,na prvnich dvou promennych PCA nepocitame jsou to jen nazvy 

summary(cars.pca)   # 1. PCA : vlastni hodnota: 6.65 (60.5%)
                    # 2. PCA : vlastni hodnota: 2.14 (19.5%)
                    # .......

#  ------------------------------------------------------------------------

#  Pocet os, ktere budeme interpreovat 
  #       a) Kaiserova-Guttmanova kriteria
  
eig<-cars.pca$CA$eig # ziskani vlastnich hodnot
eig[eig > mean(eig)]  # 2 osy -> vycerpacajici 80.0% variability z dat

#       a) Broken stick model
n<-length (eig)
bsm<-data.frame(j=seq(1:n), p=0)
bsm$p<-1/n
for (i in 2:n) {
  bsm$p[i] = bsm$p[i-1]+ (1/(n+1-i))
}
bsm$p<-100*bsm$p/n
bsm

barplot (t(cbind(100*eig/sum(eig),bsm$p[n:1])), beside=TRUE, mail="% variability", col=(c("bisque",2)), las=2)
legend ("topright", c("% variability","Broken stick model"), pch=15, col=(c("bisque",2)), bty = "n")
# take vede na 2 osy

# vykreslime graficky
par(mfrow=c(2,1))
barplot(eig, main="Vlastni hodnoty", col="bisque", las=2)
abline(h=mean(eig), col="red")
legend ("topright", "Prumerna vlastni hodnota", lwd=1, col=2, bty = "n")

barplot (t(cbind(100*eig/sum(eig),bsm$p[n:1])), beside=TRUE, mail="% variability", col=(c("bisque",2)), las=2)
legend ("topright", c("% variability","Broken stick model"), pch=15, col=(c("bisque",2)), bty = "n")

## Obe dve metody ukayuji na dve hlavni komponenty-> budeme tedy interpretovat prvni dve halvi komponenty


# vykresleni biplotu
par(mfrow=c(1,2))
biplot(cars.pca, scaling = 1, main= "PCA - skalovani 1", type = c("text")) # aby to vykreslovalo popiskz je potreba pridat argument "type = c("text")"
biplot(cars.pca, main= "PCA - skalovani 2", type = c("text"))


#  ------------------------------------------------------------------------

# Priklad 2 ---------------------------------------------------------------

# Provedte PCA na datech abundanci ryby.csv

ryby<-read.table('ryby.csv', sep=";", header=TRUE, dec=".")

# aby bylo mozne pocitat PCA je potreba data transformovat pomoci Hellingerovy transformace
ryby.stan<- decostand(ryby, "hellinger")
ryby.pca<- rda(ryby.stan)
ryby.pca

eig<-ryby.pca$CA$eig # graf vlastnich hodntot, slouzi ke stanoveni poctu interpetovanych hlavnich komponent
evplot(eig)

cleanplot.pca(ryby.pca)

